def function(inputdic, inputgt, outputgt):
    palabras = str(inputdic["input"])
    return {"output":palabras.upper()}


iclass = "PythonTransformer"
inputdic = {"input":"cstring"}
outputdic = {"output":"cstring"}
